#include <iostream>
#include <vector>
#include <string>
#include "svg.h"

using namespace std;

void svg_begin(double width, double height) {
    cout << "<?xml version='1.0' encoding='UTF-8'?>\n";
    cout << "<svg "
         << "width='" << width << "' "
         << "height='" << height << "' "
         << "viewBox='0 0 " << width << " " << height << "' "
         << "xmlns='http://www.w3.org/2000/svg'>\n";
}

void svg_end() {
    cout << "</svg>\n";
}

void svg_text(double left, double baseline, string text) {
    cout << "<text x='" << left << "' y='" << baseline << "'>" << text << "</text>\n";
}

void svg_rect(double x, double y, double width, double height, string stroke = "black", string fill = "black") {
    cout << "<rect x='" << x << "' y='" << y << "' width='" << width << "' height='" << height
         << "' stroke='" << stroke << "' fill='" << fill << "' />\n";
}

void show_histogram_svg(const vector<size_t>& bins) {
    if (bins.empty()) {
        cerr << "Error: Empty bins vector\n";
        return;
    }

    const auto IMAGE_WIDTH = 400;
    const auto IMAGE_HEIGHT = 300;
    const auto TEXT_LEFT = 20;
    const auto TEXT_BASELINE = 20;
    const auto TEXT_WIDTH = 50;
    const auto BIN_HEIGHT = 30;
    const auto MAX_WIDTH = IMAGE_WIDTH - TEXT_WIDTH;

    svg_begin(IMAGE_WIDTH, IMAGE_HEIGHT);

    double top = 0;
    size_t max_count = bins[0];
    double sum = 0;
    for (size_t bin : bins) {
        if (max_count < bin) {
            max_count = bin;
        }
        sum += bin;
    }
    double average = sum / bins.size();

    for (size_t bin : bins) {
        const double bin_width = MAX_WIDTH * (static_cast<double>(bin) / max_count);
        svg_text(TEXT_LEFT, top + TEXT_BASELINE, to_string(bin));
        string fill_color = (bin > average) ? "red" : "green";
        svg_rect(TEXT_WIDTH, top, bin_width, BIN_HEIGHT, "black", fill_color);
        top += BIN_HEIGHT;
    }

    svg_end();
}
